/******************************************************************************
 *
 * Copyright:
 *    (C) 2005 Embedded Artists AB
 *
 * File:
 *    i2c.c
 *
 * Description:
 *    Implements a generic I2C function library for the LPC2xxx processor.
 *
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include "i2c.h"
#include <lpc2xxx.h>
#include <config.h>


/******************************************************************************
 * Defines and typedefs
 *****************************************************************************/
#define CRYSTAL_FREQUENCY FOSC
#define PLL_FACTOR        PLL_MUL
#define VPBDIV_FACTOR     PBSD

//different modes
#define I2C_MODE_ACK0 0
#define I2C_MODE_ACK1 1
#define I2C_MODE_READ 2

/******************************************************************************
 *
 * Description:
 *    Read a character. I2C master mode is used.
 *    This function is also used to prepare if the master shall generate
 *    acknowledge or not acknowledge.
 *
 * Params:
 *    [in]  mode  - I2C_MODE_ACK0 Set ACK=0. Slave sends next byte
 *                  I2C_MODE_ACK1 Set ACK=1. Slave sends last byte
 *                  I2C_MODE_READ Read data from data register
 *    [out] pData - a pointer to where the data shall be saved.
 *
 * Returns:
 *    I2C_CODE_OK    - successful
 *    I2C_CODE_EMPTY - no data is available
 *
 *****************************************************************************/
tS8
i2cGetChar(tU8  mode,
           tU8* pData)
{
	tS8 retCode = I2C_CODE_OK;

	if(mode == I2C_MODE_ACK0)
	{
		//the operation mode is changed from master transmit to master receive
		//set ACK=0 (informs slave to send next byte)
		I2C_I2CONSET |= 0x04;   //AA=1
		I2C_I2CONCLR = 0x08;    //clear SI flag
	}
	else if(mode == I2C_MODE_ACK1)
	{
		//set ACK=1 (informs slave to send last byte)
		I2C_I2CONCLR = 0x04;
		I2C_I2CONCLR = 0x08;   //clear SI flag
	}
	else if(mode == I2C_MODE_READ)
	{
		//check if I2C Data register can be accessed
		if((I2C_I2CONSET & 0x08) != 0)    //SI = 1
		{
			//read data
			*pData = (tU8)I2C_I2DAT;
		}
		else
		{
			//No data available
			retCode = I2C_CODE_EMPTY;
		}
	}
	return retCode;
}


/******************************************************************************
 *
 * Description:
 *    Read a specified number of bytes from the I2C network.
 *
 *    Note: No start condition is generated in the beginningis must be
 *          done before calling this function.
 *
 * Params:
 *    [in] addr - address
 *    [in] pBuf - receive buffer
 *    [in] len  - number of bytes to receive
 *
 * Returns:
 *    I2C_CODE_OK or I2C status code
 *
 *****************************************************************************/
tS8
i2cRead(tU8  addr,
        tU8* pBuf,
        tU16 len)
{
	tS8 retCode = 0;
	tU8 status  = 0;
	tU8 i       = 0;

	// Add read bit to slave address
	addr |= 1;

	//write SLA+R
	retCode = i2cPutChar(addr);
	while(retCode == (tS8)I2C_CODE_BUSY)
		retCode = i2cPutChar(addr);

	if(retCode == I2C_CODE_OK )
	{
		//wait until address transmitted and receive data
		for(i = 1; i <= len; i++ )
		{
			//wait until data transmitted
			while(1)
			{
				//get new status
				status = i2cCheckStatus();

				/*
				 * SLA+R transmitted, ACK received or
				 * SLA+R transmitted, ACK not received
				 * data byte received in master mode, ACK transmitted
				 */
				if((status == 0x40 ) || (status == 0x48 ) || (status == 0x50 ))
				{
					//data received
					if(i == len)
					{
						//Set generate NACK
						retCode = i2cGetChar(I2C_MODE_ACK1, pBuf);
					}
					else
					{
						retCode = i2cGetChar(I2C_MODE_ACK0, pBuf);
					}

					//Read data
					retCode = i2cGetChar(I2C_MODE_READ, pBuf);
					while(retCode == (tS8)I2C_CODE_EMPTY)
						retCode = i2cGetChar(I2C_MODE_READ, pBuf);
					pBuf++;
					break;
				}

				//no relevant status information
				else if(status != 0xf8 )
				{
					//error
					i = len;
					retCode = I2C_CODE_ERROR;
					break;
				}
			}
		}
	}

	//Generate Stop condition
	i2cStop();

	return retCode;
}



