/******************************************************************************
 *
 * Copyright:
 *    (C) 2005 Embedded Artists AB
 *
 * File:
 *    i2c.c
 *
 * Description:
 *    Implements a generic I2C function library for the LPC2xxx processor.
 *
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include "i2c.h"
#include <lpc2xxx.h>
#include <config.h>


/******************************************************************************
 * Defines and typedefs
 *****************************************************************************/
#define CRYSTAL_FREQUENCY FOSC
#define PLL_FACTOR        PLL_MUL
#define VPBDIV_FACTOR     PBSD

// different modes
#define I2C_MODE_ACK0 0
#define I2C_MODE_ACK1 1
#define I2C_MODE_READ 2

/******************************************************************************
 *
 * Description:
 *    Wait after transmission of a byte until done
 *
 * Returns:
 *    I2C_CODE_OK    - successful / done
 *    I2C_CODE_ERROR - an error occured
 *
 *****************************************************************************/
tS8
i2cWaitAfterTransmit(void)
{
	tU8 status = 0;

	//wait until data transmitted
	while(1)
	{
		// get new status
		status = i2cCheckStatus();

		/*
		 * SLA+W transmitted, ACK received or
		 * data byte transmitted, ACK received
		 */
		if( (status == 0x18) || (status == 0x28) )
		{
		//data transmitted and ACK received
		return I2C_CODE_OK;
		}

		//no relevant status information
		else if(status != 0xf8 )
		{
			//error
			return I2C_CODE_ERROR;
		}
	}
}

/******************************************************************************
 *
 * Description:
 *    Sends a character on the I2C network
 *
 * Params:
 *    [in] data - the character to send
 *
 * Returns:
 *    I2C_CODE_OK   - successful
 *    I2C_CODE_BUSY - data register is not ready -> byte was not sent
 *
 *****************************************************************************/
tS8
i2cPutChar(tU8 data)
{
	tS8 retCode = 0;

	// check if I2C Data register can be accessed
	if((I2C_I2CONSET & 0x08) != 0)    //if SI = 1
	{
		/* send data */
		I2C_I2DAT    = data;
		I2C_I2CONCLR = 0x08;       //clear SI flag
		retCode    = I2C_CODE_OK;
	}
	else
	{
		// data register not ready
		retCode = I2C_CODE_BUSY;
	}

	return retCode;
}

/******************************************************************************
 *
 * Description:
 *    Sends a character on the I2C network and wait until done
 *
 * Params:
 *    [in] data - the character to send
 *
 * Returns:
 *    I2C_CODE_OK    - successful
 *    I2C_CODE_BUSY  - data register is not ready -> byte was not sent
 *    I2C_CODE_ERROR - an error occured
 *
 *****************************************************************************/
tS8
i2cPutCharAndWait(tU8 data)
{
	tS8 retCode;

	retCode = i2cPutChar(data);
	while(retCode == (tS8)I2C_CODE_BUSY)
		retCode = i2cPutChar(data);

	if(retCode == I2C_CODE_OK)
		retCode = i2cWaitAfterTransmit();

	return retCode;
}


/******************************************************************************
 *
 * Description:
 *    Sends commands and data on the I2C network.
 *
 *    Note: No stop condition is generated after the transmission - this must
 *          be done after calling this function.
 *
 * Params:
 *    [in] addr     - address
 *    [in] pCmd     - control words to transmit
 *    [in] lenCmd   - number of command bytes to transmit
 *    [in] pData    - data to transmit
 *    [in] lenData  - number of data bytes to transmit
 *
 * Returns:
 *    I2C_CODE_OK    - successful
 *    I2C_CODE_ERROR - an error occured
 *
 *****************************************************************************/
tS8
i2cWrite(tU8  addr,
         tU8  *pCmd,
         tU16 lenCmd,
         tU8* pData,
         tU16 lenData)
{
	tS8 retCode = 0;
	tU8 i       = 0;

	do
	{
		// generate Start condition
		retCode = i2cStart();
		if(retCode != I2C_CODE_OK)
			break;

		// Transmit slave address
		retCode = i2cPutCharAndWait(addr);
		if(retCode != I2C_CODE_OK)
			break;

		for(i = 0;  i< lenCmd; i++)
		{
			retCode = i2cPutCharAndWait(*pCmd);
			if(retCode != I2C_CODE_OK)
				break;
			pCmd++;
		}

		// wait until address transmitted and transmit data
		for(i = 0; i < lenData; i++)
		{
			retCode = i2cPutCharAndWait(*pData);
			if(retCode != I2C_CODE_OK)
				break;
			pData++;
		}
	} while(0); // do {} while(0); instead of multiple returns

	return retCode;
}



